package com.mortals.xhx.module.appointment.web;

import cn.hutool.core.util.StrUtil;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.IdCardTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.appointment.model.AppointmentRecordsEntity;
import com.mortals.xhx.module.person.model.PersonQuery;
import com.mortals.xhx.module.person.service.PersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.appointment.model.AppointmentConstraintEntity;
import com.mortals.xhx.module.appointment.service.AppointmentConstraintService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 预约自动签到服务约束
 *
 * @author zxfei
 * @date 2023-04-09
 */
@RestController
@RequestMapping("appointment/constraint")
public class AppointmentConstraintController extends BaseCRUDJsonBodyMappingController<AppointmentConstraintService, AppointmentConstraintEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private PersonService personService;
    @Autowired
    private UserService userService;


    public AppointmentConstraintController() {
        super.setModuleDesc("预约自动签到服务约束");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "createMethod", paramService.getParamBySecondOrganize("AppointmentConstraint", "createMethod"));
        this.addDict(model, "idCardType", IdCardTypeEnum.getEnumMap());
        this.addDict(model, "constraint", paramService.getParamBySecondOrganize("AppointmentConstraint", "constraint"));

        Map<String, String> personCollect = personService.find(new PersonQuery()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getName(), (o, n) -> n));
        this.addDict(model, "personId", personCollect);

        Map<String, String> collect = userService.find(new UserQuery(), getContext()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getRealName(), (o, n) -> n));
        this.addDict(model, "operator", collect);
        this.addDict(model, "createUserId", collect);
        super.init(model, context);
    }


    /**
     * 约束开关
     */
    @PostMapping(value = "constraint")
    public String constraintService(@RequestBody AppointmentConstraintEntity appointmentConstraintEntity) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = this.getModuleDesc() + "停止服务";
        try {
            Rest<String> rest = this.service.constraintService(appointmentConstraintEntity, getContext());
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                recordSysLog(request, busiDesc + " 【成功】");
                jsonObject.put(KEY_RESULT_DATA, model);
                jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                jsonObject.put(KEY_RESULT_MSG, rest.getMsg());
            } else {
                throw new AppException(rest.getMsg());
            }
        } catch (Exception e) {
            log.error(busiDesc, e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    @Override
    protected int doListAfter(AppointmentConstraintEntity query, Map<String, Object> model, Context context) throws AppException {
        List<AppointmentConstraintEntity> list = (List<AppointmentConstraintEntity>) model.get(KEY_RESULT_DATA);
        list.forEach(item -> {
            String maskedIdNumber = StrUtil.hide(item.getIdCardNo(), 4, item.getIdCardNo().length() - 4);
            item.setIdCardNo(maskedIdNumber);
        });
        return super.doListAfter(query, model, context);
    }
}