package com.mortals.xhx.module.care.web;
import cn.hutool.core.util.StrUtil;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.IUser;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.IdCardTypeEnum;
import com.mortals.xhx.module.appointment.model.AppointmentConstraintEntity;
import com.mortals.xhx.module.person.model.PersonQuery;
import com.mortals.xhx.module.person.service.PersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.care.model.CareConstraintEntity;
import com.mortals.xhx.module.care.service.CareConstraintService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import static com.mortals.framework.ap.SysConstains.*;
/**
*
* 主动关怀服务约束
*
* @author zxfei
* @date 2023-04-09
*/
@RestController
@RequestMapping("care/constraint")
public class CareConstraintController extends BaseCRUDJsonBodyMappingController<CareConstraintService,CareConstraintEntity,Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private PersonService personService;
    @Autowired
    private UserService userService;

    public CareConstraintController(){
        super.setModuleDesc( "主动关怀服务约束");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "createWay", paramService.getParamBySecondOrganize("CareConstraint","createWay"));
        this.addDict(model, "track", paramService.getParamBySecondOrganize("CareConstraint","track"));
        this.addDict(model, "idCardType", IdCardTypeEnum.getEnumMap());

        Map<String, String> personCollect = personService.find(new PersonQuery()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getName(), (o, n) -> n));
        this.addDict(model, "personId", personCollect);

        Map<String, String> collect = userService.find(new UserQuery(), getContext()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getRealName(), (o, n) -> n));
        this.addDict(model, "operator", collect);
        this.addDict(model, "createUserId", collect);


        super.init(model, context);
    }

    @Override
    protected int doListAfter(CareConstraintEntity query, Map<String, Object> model, Context context) throws AppException {

        List<CareConstraintEntity> list = (List<CareConstraintEntity>) model.get(KEY_RESULT_DATA);
        list.forEach(item -> {
            String maskedIdNumber = StrUtil.hide(item.getIdCard(), 4, item.getIdCard().length() - 4);
            item.setIdCard(maskedIdNumber);
        });
        return super.doListAfter(query, model, context);
    }

    @RequestMapping(value = {"track"},method = {RequestMethod.POST, RequestMethod.GET})
    public String track(Long id) {
        Map<String, Object> model = new HashMap();
        if (id == null) {
            return this.createFailJsonResp("请选择待处理" + this.getModuleDesc() + "信息");
        } else {
            JSONObject ret = new JSONObject();
            String busiDesc = "追踪/取消" + this.getModuleDesc();
            Context context = this.getContext();

            try {
                CareConstraintEntity entity = this.service.get(id, context);
                if (entity == null) {
                    throw new Exception(busiDesc + "，不存在或已被删除");
                }
                if(entity.getTrack()==1){
                    entity.setTrack(0);
                }else {
                    entity.setTrack(1);
                }
                entity.setUpdateTime(new Date());
                IUser user = this.getCurUser();
                if (user != null) {
                    entity.setUpdateUserId(user.getId());
                    entity.setUpdateUser(user.getLoginName());
                    entity.setUpdateUserName(user.getRealName());
                    entity.setUpdateUserDeptId(user.getDeptId());
                    entity.setUpdateUserDeptName(user.getDeptName());
                }
                this.service.update(entity,context);
                ret.put("data", entity);
                if (!ObjectUtils.isEmpty(context) && !ObjectUtils.isEmpty(context.getUser())) {
                    this.recordSysLog(this.request, busiDesc + " 【成功】");
                }
            } catch (Exception var8) {
                this.doException(this.request, busiDesc, model, var8);
                Object msg = model.get("message_info");
                return this.createFailJsonResp(msg == null ? "系统异常" : msg.toString());
            }

            this.init(model, context);
            ret.put("code", 1);
            ret.put("msg", model.remove("message_info"));
            ret.put("dict", model.remove("dict"));
            return ret.toJSONString();
        }
    }
}